@@ -12,9 +12,34 @@ __all__ = [
     'RangeField', 'IntegerRangeField', 'BigIntegerRangeField',
     'DecimalRangeField', 'DateTimeRangeField', 'DateRangeField',
     'FloatRangeField',
+    'RangeBoundary', 'RangeOperators',
 ]
 
 
+class RangeBoundary(models.Expression):
+    """A class that represents range boundaries."""
+    def __init__(self, inclusive_lower=True, inclusive_upper=False):
+        self.lower = '[' if inclusive_lower else '('
+        self.upper = ']' if inclusive_upper else ')'
+
+    def as_sql(self, compiler, connection):
+        return "'%s%s'" % (self.lower, self.upper), []
+
+
+class RangeOperators:
+    # https://www.postgresql.org/docs/current/functions-range.html#RANGE-OPERATORS-TABLE
+    EQUAL = '='
+    NOT_EQUAL = '<>'
+    CONTAINS = '@>'
+    CONTAINED_BY = '<@'
+    OVERLAPS = '&&'
+    FULLY_LT = '<<'
+    FULLY_GT = '>>'
+    NOT_LT = '&>'
+    NOT_GT = '&<'
+    ADJACENT_TO = '-|-'
+
+
 class RangeField(models.Field):
     empty_strings_allowed = False
 
@@ -149,12 +174,13 @@ RangeField.register_lookup(lookups.ContainedBy)
 RangeField.register_lookup(lookups.Overlap)
 
 
-class DateTimeRangeContains(models.Lookup):
+class DateTimeRangeContains(lookups.PostgresSimpleLookup):
     """
     Lookup for Date/DateTimeRange containment to cast the rhs to the correct
     type.
     """
     lookup_name = 'contains'
+    operator = RangeOperators.CONTAINS
 
     def process_rhs(self, compiler, connection):
         # Transform rhs value for db lookup.
@@ -165,9 +191,7 @@ class DateTimeRangeContains(models.Lookup):
         return super().process_rhs(compiler, connection)
 
     def as_sql(self, compiler, connection):
-        lhs, lhs_params = self.process_lhs(compiler, connection)
-        rhs, rhs_params = self.process_rhs(compiler, connection)
-        params = lhs_params + rhs_params
+        sql, params = super().as_sql(compiler, connection)
         # Cast the rhs if needed.
         cast_sql = ''
         if (
@@ -178,14 +202,14 @@ class DateTimeRangeContains(models.Lookup):
         ):
             cast_internal_type = self.lhs.output_field.base_field.get_internal_type()
             cast_sql = '::{}'.format(connection.data_types.get(cast_internal_type))
-        return '%s @> %s%s' % (lhs, rhs, cast_sql), params
+        return '%s%s' % (sql, cast_sql), params
 
 
 DateRangeField.register_lookup(DateTimeRangeContains)
 DateTimeRangeField.register_lookup(DateTimeRangeContains)
 
 
-class RangeContainedBy(models.Lookup):
+class RangeContainedBy(lookups.PostgresSimpleLookup):
     lookup_name = 'contained_by'
     type_mapping = {
         'integer': 'int4range',
@@ -194,17 +218,18 @@ class RangeContainedBy(models.Lookup):
         'date': 'daterange',
         'timestamp with time zone': 'tstzrange',
     }
+    operator = RangeOperators.CONTAINED_BY
 
-    def as_sql(self, qn, connection):
-        field = self.lhs.output_field
-        if isinstance(field, models.FloatField):
-            sql = '%s::numeric <@ %s::{}'.format(self.type_mapping[field.db_type(connection)])
-        else:
-            sql = '%s <@ %s::{}'.format(self.type_mapping[field.db_type(connection)])
-        lhs, lhs_params = self.process_lhs(qn, connection)
-        rhs, rhs_params = self.process_rhs(qn, connection)
-        params = lhs_params + rhs_params
-        return sql % (lhs, rhs), params
+    def process_rhs(self, compiler, connection):
+        rhs, rhs_params = super().process_rhs(compiler, connection)
+        cast_type = self.type_mapping[self.lhs.output_field.db_type(connection)]
+        return '%s::%s' % (rhs, cast_type), rhs_params
+
+    def process_lhs(self, compiler, connection):
+        lhs, lhs_params = super().process_lhs(compiler, connection)
+        if isinstance(self.lhs.output_field, models.FloatField):
+            lhs = '%s::numeric' % lhs
+        return lhs, lhs_params
 
     def get_prep_lookup(self):
         return RangeField().get_prep_value(self.rhs)
@@ -220,31 +245,31 @@ models.FloatField.register_lookup(RangeContainedBy)
 @RangeField.register_lookup
 class FullyLessThan(lookups.PostgresSimpleLookup):
     lookup_name = 'fully_lt'
-    operator = '<<'
+    operator = RangeOperators.FULLY_LT
 
 
 @RangeField.register_lookup
 class FullGreaterThan(lookups.PostgresSimpleLookup):
     lookup_name = 'fully_gt'
-    operator = '>>'
+    operator = RangeOperators.FULLY_GT
 
 
 @RangeField.register_lookup
 class NotLessThan(lookups.PostgresSimpleLookup):
     lookup_name = 'not_lt'
-    operator = '&>'
+    operator = RangeOperators.NOT_LT
 
 
 @RangeField.register_lookup
 class NotGreaterThan(lookups.PostgresSimpleLookup):
     lookup_name = 'not_gt'
-    operator = '&<'
+    operator = RangeOperators.NOT_GT
 
 
 @RangeField.register_lookup
 class AdjacentToLookup(lookups.PostgresSimpleLookup):
     lookup_name = 'adjacent_to'
-    operator = '-|-'
+    operator = RangeOperators.ADJACENT_TO
 
 
 @RangeField.register_lookup
