@@ -1,9 +1,12 @@
 import codecs
+import concurrent.futures
 import glob
 import os
 
 from django.core.management.base import BaseCommand, CommandError
-from django.core.management.utils import find_command, popen_wrapper
+from django.core.management.utils import (
+    find_command, is_ignored_path, popen_wrapper,
+)
 
 
 def has_bom(fn):
@@ -18,7 +21,7 @@ def is_writable(path):
     try:
         with open(path, 'a'):
             os.utime(path, None)
-    except (IOError, OSError):
+    except OSError:
         return False
     return True
 
@@ -33,22 +36,29 @@ class Command(BaseCommand):
 
     def add_arguments(self, parser):
         parser.add_argument(
-            '--locale', '-l', dest='locale', action='append', default=[],
+            '--locale', '-l', action='append', default=[],
             help='Locale(s) to process (e.g. de_AT). Default is to process all. '
                  'Can be used multiple times.',
         )
         parser.add_argument(
-            '--exclude', '-x', dest='exclude', action='append', default=[],
+            '--exclude', '-x', action='append', default=[],
             help='Locales to exclude. Default is none. Can be used multiple times.',
         )
         parser.add_argument(
             '--use-fuzzy', '-f', dest='fuzzy', action='store_true',
             help='Use fuzzy translations.',
         )
+        parser.add_argument(
+            '--ignore', '-i', action='append', dest='ignore_patterns',
+            default=[], metavar='PATTERN',
+            help='Ignore directories matching this glob-style pattern. '
+                 'Use multiple times to ignore more.',
+        )
 
     def handle(self, **options):
         locale = options['locale']
         exclude = options['exclude']
+        ignore_patterns = set(options['ignore_patterns'])
         self.verbosity = options['verbosity']
         if options['fuzzy']:
             self.program_options = self.program_options + ['-f']
@@ -65,7 +75,9 @@ class Command(BaseCommand):
         # Walk entire tree, looking for locale directories
         for dirpath, dirnames, filenames in os.walk('.', topdown=True):
             for dirname in dirnames:
-                if dirname == 'locale':
+                if is_ignored_path(os.path.normpath(os.path.join(dirpath, dirname)), ignore_patterns):
+                    dirnames.remove(dirname)
+                elif dirname == 'locale':
                     basedirs.append(os.path.join(dirpath, dirname))
 
         # Gather existing directories.
@@ -86,6 +98,7 @@ class Command(BaseCommand):
         locales = locale or all_locales
         locales = set(locales).difference(exclude)
 
+        self.has_errors = False
         for basedir in basedirs:
             if locales:
                 dirs = [os.path.join(basedir, l, 'LC_MESSAGES') for l in locales]
@@ -98,33 +111,48 @@ class Command(BaseCommand):
             if locations:
                 self.compile_messages(locations)
 
+        if self.has_errors:
+            raise CommandError('compilemessages generated one or more errors.')
+
     def compile_messages(self, locations):
         """
         Locations is a list of tuples: [(directory, file), ...]
         """
-        for i, (dirpath, f) in enumerate(locations):
-            if self.verbosity > 0:
-                self.stdout.write('processing file %s in %s\n' % (f, dirpath))
-            po_path = os.path.join(dirpath, f)
-            if has_bom(po_path):
-                raise CommandError("The %s file has a BOM (Byte Order Mark). "
-                                   "Django only supports .po files encoded in "
-                                   "UTF-8 and without any BOM." % po_path)
-            base_path = os.path.splitext(po_path)[0]
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            futures = []
+            for i, (dirpath, f) in enumerate(locations):
+                if self.verbosity > 0:
+                    self.stdout.write('processing file %s in %s\n' % (f, dirpath))
+                po_path = os.path.join(dirpath, f)
+                if has_bom(po_path):
+                    self.stderr.write(
+                        'The %s file has a BOM (Byte Order Mark). Django only '
+                        'supports .po files encoded in UTF-8 and without any BOM.' % po_path
+                    )
+                    self.has_errors = True
+                    continue
+                base_path = os.path.splitext(po_path)[0]
 
-            # Check writability on first location
-            if i == 0 and not is_writable(base_path + '.mo'):
-                self.stderr.write("The po files under %s are in a seemingly not writable location. "
-                                  "mo files will not be updated/created." % dirpath)
-                return
+                # Check writability on first location
+                if i == 0 and not is_writable(base_path + '.mo'):
+                    self.stderr.write(
+                        'The po files under %s are in a seemingly not writable location. '
+                        'mo files will not be updated/created.' % dirpath
+                    )
+                    self.has_errors = True
+                    return
 
-            args = [self.program] + self.program_options + [
-                '-o', base_path + '.mo', base_path + '.po'
-            ]
-            output, errors, status = popen_wrapper(args)
-            if status:
-                if errors:
-                    msg = "Execution of %s failed: %s" % (self.program, errors)
-                else:
-                    msg = "Execution of %s failed" % self.program
-                raise CommandError(msg)
+                args = [self.program] + self.program_options + [
+                    '-o', base_path + '.mo', base_path + '.po'
+                ]
+                futures.append(executor.submit(popen_wrapper, args))
+
+            for future in concurrent.futures.as_completed(futures):
+                output, errors, status = future.result()
+                if status:
+                    if self.verbosity > 0:
+                        if errors:
+                            self.stderr.write("Execution of %s failed: %s" % (self.program, errors))
+                        else:
+                            self.stderr.write("Execution of %s failed" % self.program)
+                    self.has_errors = True
