@@ -1,7 +1,7 @@
 """
 PostgreSQL database backend for Django.
 
-Requires psycopg 2: http://initd.org/projects/psycopg2
+Requires psycopg 2: https://www.psycopg.org/
 """
 
 import asyncio
