@@ -1,8 +1,6 @@
 "Functions that help with dynamically creating decorators for views."
 
-# For backwards compatibility in Django 2.0.
-from contextlib import ContextDecorator  # noqa
-from functools import WRAPPER_ASSIGNMENTS, partial, update_wrapper, wraps
+from functools import partial, update_wrapper, wraps
 
 
 class classonlymethod(classmethod):
@@ -113,16 +111,6 @@ def decorator_from_middleware(middleware_class):
     return make_middleware_decorator(middleware_class)()
 
 
-# Unused, for backwards compatibility in Django 2.0.
-def available_attrs(fn):
-    """
-    Return the list of functools-wrappable attributes on a callable.
-    This was required as a workaround for https://bugs.python.org/issue3445
-    under Python 2.
-    """
-    return WRAPPER_ASSIGNMENTS
-
-
 def make_middleware_decorator(middleware_class):
     def _make_decorator(*m_args, **m_kwargs):
         middleware = middleware_class(*m_args, **m_kwargs)
