@@ -5,7 +5,7 @@ that the producer of the string has already turned characters that should not
 be interpreted by the HTML engine (e.g. '<') into the appropriate entities.
 """
 
-from django.utils.functional import Promise, wraps
+from django.utils.functional import wraps
 
 
 class SafeData:
@@ -18,47 +18,26 @@ class SafeData:
         return self
 
 
-class SafeBytes(bytes, SafeData):
-    """
-    A bytes subclass that has been specifically marked as "safe" (requires no
-    further escaping) for HTML output purposes.
-
-    Kept in Django 2.0 for usage by apps supporting Python 2. Shouldn't be used
-    in Django anymore.
-    """
-    def __add__(self, rhs):
-        """
-        Concatenating a safe byte string with another safe byte string or safe
-        string is safe. Otherwise, the result is no longer safe.
-        """
-        t = super().__add__(rhs)
-        if isinstance(rhs, SafeText):
-            return SafeText(t)
-        elif isinstance(rhs, SafeBytes):
-            return SafeBytes(t)
-        return t
-
-
-class SafeText(str, SafeData):
+class SafeString(str, SafeData):
     """
     A str subclass that has been specifically marked as "safe" for HTML output
     purposes.
     """
     def __add__(self, rhs):
         """
-        Concatenating a safe string with another safe byte string or
+        Concatenating a safe string with another safe bytestring or
         safe string is safe. Otherwise, the result is no longer safe.
         """
         t = super().__add__(rhs)
         if isinstance(rhs, SafeData):
-            return SafeText(t)
+            return SafeString(t)
         return t
 
     def __str__(self):
         return self
 
 
-SafeString = SafeText
+SafeText = SafeString  # For backwards compatibility since Django 2.0.
 
 
 def _safety_decorator(safety_marker, func):
@@ -79,8 +58,6 @@ def mark_safe(s):
     """
     if hasattr(s, '__html__'):
         return s
-    if isinstance(s, (str, Promise)):
-        return SafeText(s)
     if callable(s):
         return _safety_decorator(mark_safe, s)
-    return SafeText(str(s))
+    return SafeString(s)
